from ..utils import rest
from starkcore.utils.subresource import SubResource


class BrcodeCertificate(SubResource):
    """# BrcodeCertificate object
    The BrcodeCertificate object displays the certificate information of registered SPI participants able
    to issue dynamic QR Codes.
    They are used in the validation of the URLs contained in the dynamic QR Codes.
    ## Attributes (return-only):
    - content [string]: certificate of the SPI participant in PEM format.
    - domain [string]: current active domain (URL) of the SPI participant.
    """

    def __init__(self, content=None, domain=None):
        self.content = content
        self.domain = domain


_resource = {"class": BrcodeCertificate, "name": "BrcodeCertificate"}


def query(user=None):
    """# Retrieve BrcodeCertificates
    Receive a generator of BrcodeCertificate objects.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of BrcodeCertificate objects with updated attributes
    """
    return rest.get_stream(resource=_resource, user=user)



from starkcore.utils.resource import Resource
from ..utils import rest


class IssuingBin(Resource):
    """# IssuingBin object
    The IssuingBin object displays the informations of BINs registered to your Workspace.
    They represent a group of cards that begin with the same numbers (BIN) and offer the same product to end customers.
    ## Attributes (return-only):
    - id [string]: unique BIN number registered within the card network. ex: "53810200"
    - network [string]: card network flag. ex: "mastercard"
    - settlement [string]: settlement type. ex: "credit"
    - category [string]: purchase category. ex: "prepaid"
    - client [string]: client type. ex: "business"
    - updated [datetime.datetime]: latest update datetime for the Bin. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the Bin. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id=None, updated=None, created=None, network=None, settlement=None, category=None, client=None):
        Resource.__init__(self, id=id)
        self.network = network
        self.settlement = settlement
        self.category = category
        self.client = client
        self.updated = updated
        self.created = created


_resource = {"class": IssuingBin, "name": "IssuingBin"}


def query(limit=None, user=None):
    """# Retrieve IssuingBins
    Receive a generator of IssuingBin objects previously registered in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default 100]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of IssuingBin objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        user=user,
    )


def page(limit=None, cursor=None, user=None):
    """# Retrieve paged IssuingBins
    Receive a list of up to 100 IssuingBin objects previously registered in the Stark Bank API and the cursor to the next page.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of IssuingBin objects with updated attributes
    - cursor to retrieve the next page of IssuingBin objects
    """
    return rest.get_page(
        resource=_resource,
        limit=limit,
        cursor=cursor,
        user=user,
    )



version = "0.0.6"
language = "en-US"
timeout = 15
user = None

from starkcore import Project, Organization, key, error

from . import event
from .event.__event import Event

from . import pixrequest
from .pixrequest.__pixrequest import PixRequest

from . import pixreversal
from .pixreversal.__pixreversal import PixReversal

from . import pixstatement
from .pixstatement.__pixstatement import PixStatement

from . import pixbalance
from .pixbalance.__pixbalance import PixBalance

from . import pixdirector
from .pixdirector.__pixdirector import PixDirector

from . import pixkey
from .pixkey.__pixkey import PixKey

from . import pixclaim
from .pixclaim.__pixclaim import PixClaim

from . import brcodecertificate
from .brcodecertificate.__brcodecertificate import BrcodeCertificate

from . import infractionreport
from .infractionreport.__infractionreport import InfractionReport

from . import reversalrequest
from .reversalrequest.__reversalrequest import ReversalRequest

from . import issuingauthorization
from .issuingauthorization.__issuingauthorization import IssuingAuthorization

from . import issuingbalance
from .issuingbalance.__issuingbalance import IssuingBalance

from . import creditnote
from .creditnote.__creditnote import CreditNote

from . import issuingtransaction
from .issuingtransaction.__issuingtransaction import IssuingTransaction

from . import issuingholder
from .issuingholder.__issuingholder import IssuingHolder

from . import issuingcard
from .issuingcard.__issuingcard import IssuingCard

from . import issuingpurchase
from .issuingpurchase.__issuingpurchase import IssuingPurchase

from . import issuinginvoice
from .issuinginvoice.__issuinginvoice import IssuingInvoice

from . import issuingwithdrawal
from .issuingwithdrawal.__issuingwithdrawal import IssuingWithdrawal

from . import issuingbin
from .issuingbin.__issuingbin import IssuingBin

from . import __issuingrule
from .__issuingrule.__issuingrule import IssuingRule

from .utils import endtoendid, returnid



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime
from ..utils import rest


class IssuingPurchase(Resource):
    """# IssuingPurchase object
    Displays the IssuingPurchase objects created in your Workspace.
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingPurchase is created. ex: "5656565656565656"
    - holder_name [string]: card holder name. ex: "Tony Stark"
    - card_id [string]: unique id returned when IssuingCard is created. ex: "5656565656565656"
    - card_ending [string]: last 4 digits of the card number. ex: "1234"
    - amount [integer]: IssuingPurchase value in cents. Minimum = 0. ex: 1234 (= R$ 12.34)
    - tax [integer]: IOF amount taxed for international purchases. ex: 1234 (= R$ 12.34)
    - issuer_amount [integer]: issuer amount. ex: 1234 (= R$ 12.34)
    - issuer_currency_code [string]: issuer currency code. ex: "USD"
    - issuer_currency_symbol [string]: issuer currency symbol. ex: "$"
    - merchant_amount [integer]: merchant amount. ex: 1234 (= R$ 12.34)
    - merchant_currency_code [string]: merchant currency code. ex: "USD"
    - merchant_currency_symbol [string]: merchant currency symbol. ex: "$"
    - merchant_category_code [string]: merchant category code. ex: "fastFoodRestaurants"
    - merchant_country_code [string]: merchant country code. ex: "USA"
    - acquirer_id [string]: acquirer ID. ex: "5656565656565656"
    - merchant_id [string]: merchant ID. ex: "5656565656565656"
    - merchant_name [string]: merchant name. ex: "Google Cloud Platform"
    - wallet_id [string]: virtual wallet ID. ex: "5656565656565656"
    - method_code [string]: method code. ex: "chip", "token", "server", "manual", "magstripe" or "contactless"
    - score [float]: internal score calculated for the authenticity of the purchase. None in case of insufficient data. ex: 7.6
    - issuing_transaction_ids [string]: ledger transaction ids linked to this Purchase
    - end_to_end_id [string]: central bank's unique transaction ID. ex: "E79457883202101262140HHX553UPqeq"
    - status [string]: current IssuingCard status. ex: "approved", "canceled", "denied", "confirmed" or "voided"
    - tags [string]: list of strings for tagging returned by the sub-issuer during the authorization. ex: ["travel", "food"]
    - updated [datetime.datetime]: latest update datetime for the IssuingPurchase. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the IssuingPurchase. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, holder_name, card_id, card_ending, amount, tax, issuer_amount, issuer_currency_code,
                 issuer_currency_symbol, merchant_amount, merchant_currency_code, merchant_currency_symbol,
                 merchant_category_code, merchant_country_code, acquirer_id, merchant_id, merchant_name, wallet_id,
                 method_code, score, issuing_transaction_ids, end_to_end_id, status, tags, created, updated):
        Resource.__init__(self, id=id)
        self.holder_name = holder_name
        self.card_id = card_id
        self.card_ending = card_ending
        self.amount = amount
        self.tax = tax
        self.issuer_amount = issuer_amount
        self.issuer_currency_code = issuer_currency_code
        self.issuer_currency_symbol = issuer_currency_symbol
        self.merchant_amount = merchant_amount
        self.merchant_currency_code = merchant_currency_code
        self.merchant_currency_symbol = merchant_currency_symbol
        self.merchant_category_code = merchant_category_code
        self.merchant_country_code = merchant_country_code
        self.acquirer_id = acquirer_id
        self.merchant_id = merchant_id
        self.merchant_name = merchant_name
        self.wallet_id = wallet_id
        self.method_code = method_code
        self.score = score
        self.issuing_transaction_ids = issuing_transaction_ids
        self.end_to_end_id = end_to_end_id
        self.status = status
        self.tags = tags
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": IssuingPurchase, "name": "IssuingPurchase"}


def get(id, user=None):
    """# Retrieve a specific IssuingPurchase
    Receive a single IssuingPurchase object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - IssuingPurchase object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(end_to_end_ids=None, holder_ids=None, card_ids=None, status=None, after=None, before=None, ids=None,
          limit=None, user=None):
    """# Retrieve IssuingPurchase
    Receive a generator of IssuingPurchases objects previously created in the Stark Infra API
    ## Parameters (optional):
    - end_to_end_ids [list of strings, default []]: central bank's unique transaction ID. ex: "E79457883202101262140HHX553UPqeq"
    - holder_ids [list of strings, default []]: card holder IDs. ex: ["5656565656565656", "4545454545454545"]
    - card_ids [list of strings, default []]: card  IDs. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "approved", "canceled", "denied", "confirmed" or "voided"
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - ids [list of strings, default [], default None]: purchase IDs
    - limit [integer, default 100]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of IssuingPurchase objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        end_to_end_ids=end_to_end_ids,
        holder_ids=holder_ids,
        card_ids=card_ids,
        status=status,
        after=check_datetime(after),
        before=check_datetime(before),
        ids=ids,
        limit=limit,
        user=user,
    )


def page(end_to_end_ids=None, holder_ids=None, card_ids=None, status=None, after=None, before=None, ids=None,
         cursor=None, limit=None, user=None):
    """# Retrieve paged IssuingPurchase
    Receive a list of IssuingPurchase objects previously created in the Stark Infra API and the cursor to the next page.
    ## Parameters (optional):
    - end_to_end_ids [list of strings, default []]: central bank's unique transaction ID. ex: "E79457883202101262140HHX553UPqeq"
    - holder_ids [list of strings, default []]: card holder IDs. ex: ["5656565656565656", "4545454545454545"]
    - card_ids [list of strings, default []]: card  IDs. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "approved", "canceled", "denied", "confirmed" or "voided"
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - ids [list of strings, default [], default None]: purchase IDs
    - limit [integer, default 100]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - cursor [string, default None]: cursor returned on the previous page function call
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of IssuingPurchase objects with updated attributes
    - cursor to retrieve the next page of IssuingPurchase objects
    """
    return rest.get_page(
        resource=_resource,
        end_to_end_ids=end_to_end_ids,
        holder_ids=holder_ids,
        card_ids=card_ids,
        status=status,
        after=check_datetime(after),
        before=check_datetime(before),
        ids=ids,
        cursor=cursor,
        limit=limit,
        user=user,
    )



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class InfractionReport(Resource):
    """# IssuingAuthorization object
    Infraction reports are used to report transactions that are suspected of
    fraud, to request a refund or to reverse a refund.
    When you initialize a InfractionReport, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the created object.
    ## Parameters (required):
    - reference_id [string]: end_to_end_id or return_id of the transaction being reported. ex: "E20018183202201201450u34sDGd19lz"
    - type [string]: type of infraction report. Options: "fraud", "reversal", "reversalChargeback"
    ## Parameters (optional):
    - description [string, Default None]: description for any details that can help with the infraction investigation.
    - credited_bank_code [string, Default None]: bank_code of the credited Pix participant in the reported transaction. ex: "20018183"
    ## Attributes (return-only):
    - agent [string]: Options: "reporter" if you created the InfractionReport, "reported" if you received the InfractionReport.
    - analysis [string]: analysis that led to the result.
    - bacen_id [string]: central bank's unique UUID that identifies the infraction report.
    - debited_bank_code [string]: bank_code of the debited Pix participant in the reported transaction. ex: "20018183"
    - id [string]: unique id returned when the InfractionReport is created. ex: "5656565656565656"
    - reported_by [string]: agent that reported the InfractionReport. Options: "debited", "credited".
    - result [string]: result after the analysis of the InfractionReport by the receiving party. Options: "agreed", "disagreed"
    - status [string]: current InfractionReport status. Options: "created", "failed", "delivered", "closed", "canceled".
    - created [datetime.datetime]: creation datetime for the InfractionReport. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the InfractionReport. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self,  reference_id, type, description=None, credited_bank_code=None, agent=None, analysis=None,
                 bacen_id=None, debited_bank_code=None, id=None, reported_by=None, result=None, status=None,
                 created=None, updated=None):
        Resource.__init__(self, id=id)

        self.reference_id = reference_id
        self.type = type
        self.description = description
        self.credited_bank_code = credited_bank_code
        self.agent = agent
        self.analysis = analysis
        self.bacen_id = bacen_id
        self.debited_bank_code = debited_bank_code
        self.reported_by = reported_by
        self.result = result
        self.status = status
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": InfractionReport, "name": "InfractionReport"}


def create(report, user=None):
    """# Create a InfractionReport object
    Create a InfractionReport in the Stark Infra API
    ## Parameters (optional):
    - report [InfractionReport object]: InfractionReport object to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - InfractionReport object with updated attributes.
    """
    return rest.post_single(resource=_resource, entity=report, user=user)


def get(id, user=None):
    """# Retrieve a InfractionReport object
    Retrieve the InfractionReport object linked to your Workspace in the Stark Infra API using its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656".
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - InfractionReport object that corresponds to the given id.
    """

    return rest.get_id(id=id, resource=_resource, user=user)


def query(limit=None, after=None, before=None, status=None, ids=None, type=None, user=None):
    """# Retrieve InfractionReports
    Receive a generator of InfractionReports objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. Options: "created", "failed", "delivered", "closed", "canceled".
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - type [list of strings, default None]: filter for the type of retrieved InfractionReports. Options: "fraud", "reversal", "reversalChargeback"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of InfractionReport objects with updated attributes
    """

    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        type=type,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, ids=None, type=None,
         user=None):
    """# Retrieve paged InfractionReports
    Receive a list of up to 100 InfractionReports objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call.
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. Options: "created", "failed", "delivered", "closed", "canceled".
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - type [list of strings, default None]: filter for the type of retrieved InfractionReports. Options: "fraud", "reversal", "reversalChargeback"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of InfractionReport objects with updated attributes and cursor to retrieve the next page of InfractionReport objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        type=type,
        user=user,
    )


def update(id, result, analysis=None, user=None):
    """# Update InfractionReport entity
    Respond to a received InfractionReport.
    ## Parameters (required):
    - id [string]: InfractionReport id. ex: '5656565656565656'
    - result [string]: result after the analysis of the InfractionReport. Options: "agreed", "disagreed"
    ## Parameters (optional):
    - analysis [string, default None]: analysis that led to the result.
    ## Return:
    - InfractionReport with updated attributes
    """
    payload = {
        "result": result,
        "analysis": analysis,
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def delete(id, user=None):
    """# Delete a InfractionReport entity
    Delete a InfractionReport entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - deleted InfractionReport object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from json import dumps
from starkcore.utils.resource import Resource
from starkinfra.utils.parse import parse_and_verify


class IssuingAuthorization(Resource):
    """# IssuingAuthorization object
    An IssuingAuthorization presents purchase data to be analysed and answered with an approval or a declination.
    ## Attributes:
    - end_to_end_id [string]: central bank's unique transaction ID. ex: "E79457883202101262140HHX553UPqeq"
    - amount [integer]: IssuingPurchase value in cents. Minimum = 0. ex: 1234 (= R$ 12.34)
    - tax [integer]: IOF amount taxed for international purchases. ex: 1234 (= R$ 12.34)
    - card_id [string]: unique id returned when IssuingCard is created. ex: "5656565656565656"
    - issuer_amount [integer]: issuer amount. ex: 1234 (= R$ 12.34)
    - issuer_currency_code [string]: issuer currency code. ex: "USD"
    - merchant_amount [integer]: merchant amount. ex: 1234 (= R$ 12.34)
    - merchant_currency_code [string]: merchant currency code. ex: "USD"
    - merchant_category_code [string]: merchant category code. ex: "fastFoodRestaurants"
    - merchant_country_code [string]: merchant country code. ex: "USA"
    - acquirer_id [string]: acquirer ID. ex: "5656565656565656"
    - merchant_id [string]: merchant ID. ex: "5656565656565656"
    - merchant_name [string]: merchant name. ex: "Google Cloud Platform"
    - merchant_fee [integer]: merchant fee charged. ex: 200 (= R$ 2.00)
    - wallet_id [string]: virtual wallet ID. ex: "googlePay"
    - method_code [string]: method code. ex: "chip", "token", "server", "manual", "magstripe" or "contactless"
    - score [float]: internal score calculated for the authenticity of the purchase. None in case of insufficient data. ex: 7.6
    - is_partial_allowed [bool]: true if the the merchant allows partial purchases. ex: False
    - purpose [string]: purchase purpose. ex: "purchase"
    - card_tags [list of strings]: tags of the IssuingCard responsible for this purchase. ex: ["travel", "food"]
    - holder_tags [list of strings]: tags of the IssuingHolder responsible for this purchase. ex: ["technology", "john snow"]
    """

    def __init__(self, end_to_end_id, amount, tax, card_id, issuer_amount, issuer_currency_code, merchant_amount,
                 merchant_currency_code, merchant_category_code, merchant_country_code, acquirer_id, merchant_id,
                 merchant_name, merchant_fee, wallet_id, method_code, score, is_partial_allowed, purpose, card_tags,
                 holder_tags, id):
        Resource.__init__(self, id=id)
        self.end_to_end_id = end_to_end_id
        self.amount = amount
        self.tax = tax
        self.card_id = card_id
        self.issuer_amount = issuer_amount
        self.issuer_currency_code = issuer_currency_code
        self.merchant_amount = merchant_amount
        self.merchant_currency_code = merchant_currency_code
        self.merchant_category_code = merchant_category_code
        self.merchant_country_code = merchant_country_code
        self.acquirer_id = acquirer_id
        self.merchant_id = merchant_id
        self.merchant_name = merchant_name
        self.merchant_fee = merchant_fee
        self.wallet_id = wallet_id
        self.method_code = method_code
        self.score = score
        self.is_partial_allowed = is_partial_allowed
        self.purpose = purpose
        self.card_tags = card_tags
        self.holder_tags = holder_tags


_resource = {"class": IssuingAuthorization, "name": "IssuingAuthorization"}


def parse(content, signature, user=None):
    """# Create single IssuingAuthorization from a content string
    Create a single IssuingAuthorization object received from IssuingAuthorization at the informed endpoint.
    If the provided digital signature does not check out with the StarkInfra public key, a
    stark.exception.InvalidSignatureException will be raised.
    ## Parameters (required):
    - content [string]: response content from request received at user endpoint (not parsed)
    - signature [string]: base-64 digital signature received at response header "Digital-Signature"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - Parsed IssuingAuthorization object
    """
    return parse_and_verify(
        content=content,
        signature=signature,
        user=user,
        resource=_resource,
        key="",
    )


def response(status, amount=None, reason=None, tags=None):
    """# Helps you respond IssuingAuthorization requests.
    ## Parameters (required):
    - status [string]: sub-issuer response to the authorization. ex: "accepted" or "denied"
    ## Parameters (optional):
    - amount [integer, default 0]: amount in cents that was authorized. ex: 1234 (= R$ 12.34)
    - reason [string, default ""]: denial reason. ex: "other"
    - tags [list of strings, default []]: tags to filter retrieved object. ex: ["tony", "stark"]
    ## Return:
    - Dumped JSON string that must be returned to us on the IssuingAuthorization request
    """
    return dumps({"authorization": {
        "status": status,
        "amount": amount or 0,
        "reason": reason or "",
        "tags": tags or [],
    }})



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class CreditNote(Resource):
    """# CreditNote object
    CreditNotes are used to generate CCB contracts between you and your customers.
    When you initialize a CreditNote, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the list of created objects.
    ## Parameters (required):
    - template_id [string]: ID of the contract template on which the credit note will be based. ex: template_id="0123456789101112"
    - name [string]: credit receiver's full name. ex: name="Edward Stark"
    - tax_id [string]: credit receiver's tax ID (CPF or CNPJ). ex: "20.018.183/0001-80"
    - nominal_amount [integer]: amount in cents transferred to the credit receiver, before deductions. ex: nominal_amount=11234 (= R$ 112.34)
    - scheduled [datetime.date, datetime.datetime or string, default now]: date of transfer execution. ex: scheduled=datetime(2020, 3, 10)
    - invoices [list of Invoice objects]: list of Invoice objects to be created and sent to the credit receiver. ex: invoices=[Invoice(), Invoice()]
    - transfer [Transfer object]: Transfer object to be created and sent to the credit receiver. ex: transfer=Transfer()
    - signers [list of dictionaries]: signer's name, e-mail and delivery method for the contract. ex: signers=[{"name": "Tony Stark", "contact": "tony@starkindustries.com", "method": "link"}]
    - externalId [string]: a string that must be unique among all your CreditNotes, used to avoid resource duplication. ex: "my-internal-id-123456"
    Parameters (optional):
    - rebate_amount [integer, default None]: credit analysis fee deducted from lent amount. ex: rebate_amount=11234 (= R$ 112.34)
    - tags [list of strings, default None]: list of strings for reference when searching for CreditNotes. ex: tags=["employees", "monthly"]
    Attributes (return-only):
    - id [string]: unique id returned when the CreditNote is created. ex: "5656565656565656"
    - interest [float]: yearly effective interest rate of the credit note, in percentage. ex: 12.5
    - created [datetime.datetime]: creation datetime for the CreditNote. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the CreditNote. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, template_id, name, tax_id, nominal_amount, scheduled, invoices, transfer, signers,
                 external_id, interest=None, rebate_amount=None, tags=None, created=None, updated=None, id=None):
        Resource.__init__(self, id=id)

        self.template_id = template_id
        self.name = name
        self.tax_id = tax_id
        self.nominal_amount = nominal_amount
        self.scheduled = scheduled
        self.invoices = invoices
        self.transfer = transfer
        self.signers = signers
        self.external_id = external_id
        self.interest = interest
        self.rebate_amount = rebate_amount
        self.tags = tags
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": CreditNote, "name": "CreditNote"}


def create(notes, user=None):
    """# Create CreditNotes
    Send a list of CreditNote objects for creation in the Stark Bank API
    ## Parameters (required):
    - notes [list of CreditNote objects]: list of CreditNote objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of CreditNote objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=notes, user=user)


def get(id, user=None):
    """# Retrieve a specific CreditNote
    Receive a single CreditNote object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - CreditNote object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve CreditNotes
    Receive a generator of CreditNote objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default 100]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: filter for status of retrieved objects. ex: "paid" or "registered"
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of CreditNote objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve paged CreditNotes
    Receive a list of up to 100 CreditNote objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: filter for status of retrieved objects. ex: "paid" or "registered"
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of CreditNote objects with updated attributes
    - cursor to retrieve the next page of CreditNote objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )

def delete(id, user=None):
    """# Cancel a Credit Note entity
    Cancel a Credit Note entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: Credit Note unique id. ex: "6306109539221504"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - deleted Credit Note object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from starkcore.utils.checks import check_datetime, check_date
from ..__infractionreport import _resource as _infractionreport_resource
from starkcore.utils.resource import Resource
from starkcore.utils.api import from_api_json
from ...utils import rest


class Log(Resource):
    """# InfractionReport.Log object
    Every time a InfractionReport entity is modified, a corresponding InfractionReport.Log
    is generated for the entity. This log is never generated by the user.
    ## Attributes:
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - type [string]: type of the InfractionReport event which triggered the log creation. ex: "created", "registered", "updated", "failed", "canceling" and "canceled".
    - errors [list of strings]: list of errors linked to this InfractionReport event
    - report [InfractionReport]: InfractionReport entity to which the log refers to.
    """
    def __init__(self, id, created, type, errors, report):
        Resource.__init__(self, id=id)

        self.created = check_datetime(created)
        self.type = type
        self.errors = errors
        self.report = from_api_json(_infractionreport_resource, report)


_resource = {"class": Log, "name": "InfractionReportLog"}


def get(id, user=None):
    """# Retrieve a specific InfractionReport.Log
    Receive a single InfractionReport.Log object previously created by the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - InfractionReport.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(ids=None, limit=None, after=None, before=None, types=None, report_ids=None, user=None):
    """# Retrieve InfractionReport.Logs
    Receive a generator of InfractionReport.Log objects previously created in the Stark Infra API
    ## Parameters (optional):
    - ids [list of strings, default None]: Log ids to filter InfractionReport Logs. ex: ["5656565656565656"]
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter retrieved objects by types. ex: "success" or "failed"
    - report_ids [list of strings, default None]: list of InfractionReport IDs to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of InfractionReport.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        report_ids=report_ids,
        user=user,
    )


def page(cursor=None, ids=None, limit=None, after=None, before=None, types=None, report_ids=None, user=None):
    """# Retrieve paged InfractionReport.Logs
    Receive a list of up to 100 InfractionReport.Log objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your reports.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - ids [list of strings, default None]: Log ids to filter InfractionReport Logs. ex: ["5656565656565656"]
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter retrieved objects by types. ex: "success" or "failed"
    - report_ids [list of strings, default None]: list of InfractionReport ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of InfractionReport.Log objects with updated attributes
    - cursor to retrieve the next page of InfractionReport.Log objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        report_ids=report_ids,
        user=user,
    )



from starkcore.utils.resource import Resource


class IssuingRule(Resource):

    """# IssuingRule object
    The IssuingRule object displays the spending rules of IssuingCards and IssuingHolders created in your Workspace.
    ## Parameters (required):
    - name [string]: rule name. ex: "Travel" or "Food"
    - amount [integer]: maximum amount that can be spent in the informed interval. ex: 200000 (= R$ 2000.00)
    - interval [string]: interval after which the rule amount counter will be reset to 0. ex: "instant", "day", "week", "month", "year" or "lifetime"
    ## Parameters (optional):
    - currency_code [string, default "BRL"]: code of the currency that the rule amount refers to. ex: "BRL" or "USD"
    - categories [list of strings, default []]: merchant categories accepted by the rule. ex: ["eatingPlacesRestaurants", "travelAgenciesTourOperators"]
    - countries [list of strings, default []]: countries accepted by the rule. ex: ["BRA", "USA"]
    - methods [list of strings, default []]: card purchase methods accepted by the rule. ex: ["contactless", "manual"]
    ## Attributes (expanded return-only):
    - counter_amount [integer]: current rule spent amount. ex: 1000
    - currency_symbol [string]: currency symbol. ex: "R$"
    - currency_name [string]: currency name. ex: "Brazilian Real"
    ## Attributes (return-only):
    - id [string]: unique id returned when Rule is created. ex: "5656565656565656"
    """

    def __init__(self, id=None, name=None, interval=None, amount=None, currency_code=None, counter_amount=None,
                 currency_name=None, currency_symbol=None, categories=None, countries=None, methods=None):
        Resource.__init__(self, id=id)
        self.name = name
        self.interval = interval
        self.amount = amount
        self.currency_code = currency_code
        self.counter_amount = counter_amount
        self.currency_name = currency_name
        self.currency_symbol = currency_symbol
        self.categories = categories
        self.countries = countries
        self.methods = methods



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_date


class ReversalRequest(Resource):
    """# ReversalRequest object
    A reversal request can be created when fraud is detected on a transaction or a system malfunction
    results in an erroneous transaction.
    It notifies another participant of your request to reverse the payment they have received.
    When you initialize a ReversalRequest, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the created object.
    ## Parameters (required):
    - amount [integer]: amount in cents to be reversed. ex: 11234 (= R$ 112.34)
    - reference_id [string]: end_to_end_id or return_id of the transaction to be reversed. ex: "E20018183202201201450u34sDGd19lz"
    - reason [string]: reason why the reversal was requested. Options: "fraud", "flaw", "reversalChargeback"
    ## Parameters (optional):
    - description [string, default None]: description for the ReversalRequest.
    ## Attributes (return-only):
    - analysis [string]: analysis that led to the result.
    - bacen_id [string]: central bank's unique UUID that identifies the ReversalRequest.
    - sender_bank_code [string]: bank_code of the Pix participant that created the ReversalRequest. ex: "20018183"
    - receiver_bank_code [string]: bank_code of the Pix participant that received the ReversalRequest. ex: "20018183"
    - rejection_reason [string]: reason for the rejection of the reversal request. Options: "noBalance", "accountClosed", "unableToReverse"
    - reversal_reference_id [string]: return id of the reversal transaction. ex: "D20018183202202030109X3OoBHG74wo".
    - id [string]: unique id returned when the ReversalRequest is created. ex: "5656565656565656"
    - result [string]: result after the analysis of the ReversalRequest by the receiving party. Options: "rejected", "accepted", "partiallyAccepted"
    - status [string]: current ReversalRequest status. Options: "created", "failed", "delivered", "closed", "canceled".
    - created [datetime.datetime]: creation datetime for the ReversalRequest. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the ReversalRequest. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self,  amount, reference_id, reason, description=None, analysis=None, bacen_id=None,
                 sender_bank_code=None, receiver_bank_code=None, rejection_reason=None, reversal_reference_id=None, id=None,
                 result=None, status=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.reference_id = reference_id
        self.reason = reason
        self.description = description
        self.analysis = analysis
        self.bacen_id = bacen_id
        self.sender_bank_code = sender_bank_code
        self.receiver_bank_code = receiver_bank_code
        self.rejection_reason = rejection_reason
        self.reversal_reference_id = reversal_reference_id
        self.result = result
        self.status = status
        self.created = created
        self.updated = updated


_resource = {"class": ReversalRequest, "name": "ReversalRequest"}


def create(request, user=None):
    """# Create a ReversalRequest object
    Create a ReversalRequest in the Stark Infra API
    ## Parameters (optional):
    - request [ReversalRequest object]: ReversalRequest object to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - ReversalRequest object with updated attributes.
    """
    return rest.post_single(resource=_resource, entity=request, user=user)


def get(id, user=None):
    """# Retrieve a ReversalRequest object
    Retrieve the ReversalRequest object linked to your Workspace in the Stark Infra API using its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656".
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - ReversalRequest object that corresponds to the given id.
    """

    return rest.get_id(id=id, resource=_resource, user=user)


def query(limit=None, after=None, before=None, status=None, ids=None, user=None):
    """# Retrieve ReversalRequests
    Receive a generator of ReversalRequests objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. Options: "created", "failed", "delivered", "closed", "canceled".
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of ReversalRequest objects with updated attributes
    """

    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, ids=None, user=None):
    """# Retrieve ReversalRequests
    Receive a generator of ReversalRequests objects previously created in the Stark Infra API
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call.
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. Options: "created", "failed", "delivered", "closed", "canceled".
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - type [list of strings, default None]: filter for the type of retrieved ReversalRequests. Options: "fraud", "reversal", "reversalChargeback"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - cursor to retrieve the next page of ReversalRequest objects
    - generator of ReversalRequest objects with updated attributes
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        user=user,
    )


def update(id, result, rejection_reason=None, reversal_reference_id=None, analysis=None, user=None):
    """# Update ReversalRequest entity
    Respond to a received ReversalRequest.
    ## Parameters (required):
    - id [string]: ReversalRequest id. ex: '5656565656565656'
    - result [string]: result after the analysis of the ReversalRequest. Options: "rejected", "accepted", "partiallyAccepted".
    ## Parameters (conditionally required):
    - rejection_reason [string, default None]: if the ReversalRequest is rejected a reason is required. Options: "noBalance", "accountClosed", "unableToReverse",
    - reversal_reference_id [string, default None]: return_id of the reversal transaction. ex: "D20018183202201201450u34sDGd19lz"
    ## Parameters (optional):
    - analysis [string, default None]: description of the analysis that led to the result.
    ## Return:
    - ReversalRequest with updated attributes
    """
    payload = {
        "result": result,
        "rejection_reason": rejection_reason,
        "reversal_reference_id": reversal_reference_id,
        "analysis": analysis
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def delete(id, user=None):
    """# Delete a ReversalRequest entity
    Delete a ReversalRequest entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - deleted ReversalRequest object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime


class IssuingCard(Resource):

    """# IssuingCard object
    The IssuingCard object displays the information of the cards created in your Workspace.
    Sensitive information will only be returned when the "expand" parameter is used, to avoid security concerns.
    ## Parameters (required):
    - holder_name [string]: card holder name. ex: "Tony Stark"
    - holder_tax_id [string]: card holder tax ID. ex: "012.345.678-90"
    - holder_external_id [string] card holder unique id, generated by the user to avoid duplicated holders. ex: "my-entity/123"
    ## Parameters (optional):
    - display_name [string, default None]: card displayed name. ex: "ANTHONY STARK"
    - rules [list of IssuingRule, default None]: [EXPANDABLE] list of card spending rules.
    - bin_id [string, default None]: BIN ID to which the card is bound. ex: "53810200"
    - tags [list of strings]: list of strings for tagging. ex: ["travel", "food"]
    - street_line_1 [string, default None]: card holder main address. ex: "Av. Paulista, 200"
    - street_line_2 [string, default None]: card holder address complement. ex: "Apto. 123"
    - district [string]: card holder address district / neighbourhood. ex: "Bela Vista"
    - city [string, default None]: card holder address city. ex: "Rio de Janeiro"
    - state_code [string, default None]: card holder address state. ex: "GO"
    - zip_code [string]: card holder address zip code. ex: "01311-200"
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingCard is created. ex: "5656565656565656"
    - holder_id [string]: card holder unique id. ex: "5656565656565656"
    - type [string]: card type. ex: "virtual"
    - status [string]: current IssuingCard status. ex: "canceled" or "active"
    - number [string]: [EXPANDABLE] masked card number. Expand to unmask the value. ex: "123".
    - security_code [string]: [EXPANDABLE] masked card verification value (cvv). Expand to unmask the value. ex: "123".
    - expiration [string]: [EXPANDABLE] masked card expiration datetime. Expand to unmask the value. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0).
    - updated [datetime.datetime]: latest update datetime for the IssuingCard. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the IssuingCard. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, holder_name, holder_tax_id, holder_external_id, id=None, holder_id=None, type=None, display_name=None, status=None,
                 rules=None, bin_id=None, street_line_1=None, street_line_2=None, district=None, city=None, state_code=None, zip_code=None,
                 tags=None, created=None, updated=None, number=None, security_code=None, expiration=None):
        Resource.__init__(self, id=id)
        self.id = id
        self.holder_id = holder_id
        self.holder_name = holder_name
        self.holder_tax_id = holder_tax_id
        self.holder_external_id = holder_external_id
        self.type = type
        self.display_name = display_name
        self.status = status
        self.rules = rules
        self.bin_id = bin_id
        self.street_line_1 = street_line_1
        self.street_line_2 = street_line_2
        self.district = district
        self.city = city
        self.state_code = state_code
        self.zip_code = zip_code
        self.tags = tags
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)
        self.number = number
        self.security_code = security_code
        self.expiration = expiration


_resource = {"class": IssuingCard, "name": "IssuingCard"}


def create(cards, expand=None, user=None):
    """# Create IssuingCards
    Send a list of IssuingCard objects for creation in the Stark Infra API
    ## Parameters (required):
    - cards [list of IssuingCard objects]: list of IssuingCard objects to be created in the API
    ## Parameters (optional):
    - expand [list of strings, default []]: fields to expand information. ex: ["rules", "security_code", "number", "expiration"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of IssuingCard objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=cards, expand=expand, user=user)


def query(status=None, types=None, holder_ids=None, after=None, before=None, tags=None, ids=None, limit=None, expand=None, user=None):
    """# Retrieve IssuingCards
    Receive a generator of IssuingCards objects previously created in the Stark Infra API
    ## Parameters (optional):
    - status [string, default None]: filter for status of retrieved objects. ex: "paid" or "registered"
    - types [list of strings, default None]: card type. ex: ["virtual"]
    - holder_ids [list of strings]: card holder IDs. ex: ["5656565656565656", "4545454545454545"]
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - limit [integer, default 100]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - expand [list of strings, default []]: fields to expand information. ex: ["rules", "security_code", "number", "expiration"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of IssuingCards objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        status=status,
        types=types,
        holder_ids=holder_ids,
        after=check_datetime(after),
        before=check_datetime(before),
        tags=tags,
        ids=ids,
        limit=limit,
        expand=expand,
        user=user,
    )


def page(status=None, types=None, holder_ids=None, after=None, before=None, tags=None, ids=None,
         limit=None, cursor=None, expand=None, user=None):
    """# Retrieve paged IssuingCards
    Receive a list of IssuingCards objects previously created in the Stark Infra API and the cursor to the next page.
    ## Parameters (optional):
    - status [string, default None]: filter for status of retrieved objects. ex: "paid" or "registered"
    - types [list of strings, default None]: card type. ex: ["virtual"]
    - holder_ids [list of strings, default None]: card holder IDs. ex: ["5656565656565656", "4545454545454545"]
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - limit [integer, default 100]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - cursor [string, default None]: cursor returned on the previous page function call
    - expand [list of strings, default []]: fields to expand information. ex: ["rules", "security_code", "number", "expiration"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of IssuingCards objects with updated attributes
    - cursor to retrieve the next page of IssuingCards objects
    """
    return rest.get_page(
        resource=_resource,
        status=status,
        types=types,
        holder_ids=holder_ids,
        after=check_datetime(after),
        before=check_datetime(before),
        tags=tags,
        ids=ids,
        limit=limit,
        cursor=cursor,
        expand=expand,
        user=user,
    )


def get(id, expand=None, user=None):
    """# Retrieve a specific IssuingCards
    Receive a single IssuingCards object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - expand [list of strings, default None]: fields to expand information. ex: ["rules"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - IssuingCards object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, expand=expand, user=user)


def update(id, status=None, display_name=None, rules=None, tags=None, user=None):
    """# Update IssuingCard entity
    Update an IssuingCard by passing id.
    ## Parameters (required):
    - id [string]: IssuingCard id. ex: '5656565656565656'
    ## Parameters (optional):
    - status [string]: You may block the IssuingCard by passing 'blocked' in the status
    - display_name [string, default None]: card displayed name
    - rules [list of dictionaries, default None]: list of dictionaries with "amount": int, "currencyCode": string, "id": string, "interval": string, "name": string pairs.
    - tags [list of strings]: list of strings for tagging
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - target IssuingCard with updated attributes
    """
    payload = {
        "status": status,
        "display_name": display_name,
        "rules": rules,
        "tags": tags,
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def delete(id, user=None):
    """# Delete an IssuingCard entity
    Delete an IssuingCard entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: IssuingCard unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - deleted IssuingCard object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)
